using System;
using System.Drawing;
using System.Drawing.Design;
using System.Collections;
using System.ComponentModel;
using System.Collections.Specialized;
using System.Windows.Forms;
using System.Windows.Forms.Design;
using System.Security;
using System.Security.Permissions;
using System.Diagnostics;

// using CaveArt.Common.Configuration;		// Const.CurrencyFormat

#if USE_CURRENY_EDIT
using CaveArt.UIcomp.Edits;
#endif

namespace CaveArt.UIcomp
{
	#region NoEditor
	public class NoEditor : UITypeEditor
	{
		public override UITypeEditorEditStyle GetEditStyle(ITypeDescriptorContext context)
		{
			return UITypeEditorEditStyle.None;
		}
	}
	#endregion

	#region FileFilterEditor
	public class FileFilterEditor : UITypeEditor
	{
		[System.Security.Permissions.PermissionSet(System.Security.Permissions.SecurityAction.Demand, Name="FullTrust")]
		public override UITypeEditorEditStyle GetEditStyle(ITypeDescriptorContext context)
		{
			if (context != null && 
				context.Instance != null)
			{
				return UITypeEditorEditStyle.Modal;
			}

			return UITypeEditorEditStyle.None;
		}

		[RefreshProperties(RefreshProperties.All)]
		[System.Security.Permissions.PermissionSet(System.Security.Permissions.SecurityAction.Demand, Name="FullTrust")]
		public override object EditValue(ITypeDescriptorContext context, System.IServiceProvider provider, object value)
		{
			if (context == null || 
				provider == null || 
				context.Instance == null)
			{
				return base.EditValue(provider, value);
			}

			FileDialog dlg;

			if (context.PropertyDescriptor.Attributes[typeof(SaveFileAttribute)] == null)
				dlg = new OpenFileDialog();
			else
				dlg = new SaveFileDialog();

			dlg.Title = "Select " + context.PropertyDescriptor.DisplayName;
			dlg.FileName = value.ToString();

			FileDialogFilterAttribute filterAtt = (FileDialogFilterAttribute) context.PropertyDescriptor.Attributes[typeof(FileDialogFilterAttribute)];
			
			if (filterAtt != null)
			{
				dlg.Filter = filterAtt.Filter;
			}

			if (dlg.ShowDialog() == DialogResult.OK)
			{
				value = dlg.FileName;
			}

			dlg.Dispose();

			return value;
		}
	}
	
	[AttributeUsage(AttributeTargets.Property)]
	public class FileDialogFilterAttribute : Attribute
	{
		private string m_filter;

		///-----------------------------------------------------------------------------
		/// Define a filter for the FileFilterEditor.
		/// The filter to use in the file dialog in FileFilterEditor. 
		/// The following is an example of a filter string: "Text files (*.txt)|*.txt|All files (*.*)|*.*"
		/// [Vadim] 30/12/2003 Created
		///-----------------------------------------------------------------------------
		public FileDialogFilterAttribute(string filter)
		{
			m_filter = filter;
		}

		///-----------------------------------------------------------------------------
		/// The filter to use in the file dialog in FileFilterEditor.
		/// The following is an example of a filter string: "Text files (*.txt)|*.txt|All files (*.*)|*.*"
		/// [Vadim] 30/12/2003 Created
		///-----------------------------------------------------------------------------
		public string Filter
		{ 
			get { return m_filter; } 
		}
	}

	[AttributeUsage(AttributeTargets.Property)]
	public class SaveFileAttribute : Attribute
	{
		public SaveFileAttribute(){}
	}
	
	[AttributeUsage(AttributeTargets.Property)]
	public class OpenFileAttribute : Attribute
	{
		public OpenFileAttribute(){}
	}
	#endregion

	#region Decimal Type Editor
	public class DecimalTypeConverter : TypeConverter
	{
		public override bool CanConvertFrom(ITypeDescriptorContext context, Type sourceType)
		{
			if (sourceType == typeof(string))
			{
				return true;
			}
			return base.CanConvertFrom (context, sourceType);
		}

		public override bool CanConvertTo(ITypeDescriptorContext context, Type destinationType)
		{
			if (destinationType == typeof(string))
			{
				return true;
			}
			return base.CanConvertTo (context, destinationType);
		}

		public override object ConvertFrom(ITypeDescriptorContext context, System.Globalization.CultureInfo culture, object value)
		{
			if(value.GetType() == typeof(string))
			{
				try
				{
					return decimal.Parse(String.Format(Const.CurrencyFormat, value));
				}
				catch
				{
					throw new InvalidCastException();
				}
			}

			return base.ConvertFrom (context, culture, value);
		}

		public override object ConvertTo(ITypeDescriptorContext context, System.Globalization.CultureInfo culture, object value, Type destinationType)
		{
			if ( destinationType == typeof(string) && value is decimal )
			{	
				return String.Format(Const.CurrencyFormat, value);
			}

			return base.ConvertTo (context, culture, value, destinationType);
		}		
	}
	#endregion

	#region Date Type Converter
	public class DateTypeConverter : CaveArtTypeConverter
	{
		public override bool CanConvertFrom(ITypeDescriptorContext context, Type sourceType)
		{
			if (sourceType == typeof(string))
			{
				return true;
			}
			return base.CanConvertFrom (context, sourceType);
		}

		public override bool CanConvertTo(ITypeDescriptorContext context, Type destinationType)
		{
			if (destinationType == typeof(string))
			{
				return true;
			}
			return base.CanConvertTo (context, destinationType);
		}

		public override object ConvertFrom(ITypeDescriptorContext context, System.Globalization.CultureInfo culture, object value)
		{
			if ( value.GetType() == typeof(string) )
			{
				try
				{
					if ( Const.NotSpecifiedString == (string) value )
					{
						return Const.UNDEFINED_TIME;
					}
					else 
					{
						return DateTime.Parse( (string) value );
					}
				}
				catch
				{
					throw new InvalidCastException();
				}
			}

			return base.ConvertFrom(context, culture, value);
		}

		public override object ConvertTo(ITypeDescriptorContext context, System.Globalization.CultureInfo culture, object value, Type destinationType)
		{
			if ( destinationType == typeof(string) && value is DateTime )
			{				
				if ((DateTime) value == Const.UNDEFINED_TIME)
				{
					return Const.NotSpecifiedString;
				}
				else 
				{
					return ((DateTime) value).ToString(System.Globalization.DateTimeFormatInfo.CurrentInfo.ShortDatePattern);
				}
			}

			return base.ConvertTo (context, culture, value, destinationType);
		}		
	}
	#endregion

	#region Expiry Type Converter
	public class ExpiryTypeConverter : CaveArtTypeConverter
	{
		public override bool CanConvertFrom(ITypeDescriptorContext context, Type sourceType)
		{
			if (sourceType == typeof(string))
			{
				return true;
			}
			return base.CanConvertFrom (context, sourceType);
		}

		public override bool CanConvertTo(ITypeDescriptorContext context, Type destinationType)
		{
			if (destinationType == typeof(string))
			{
				return true;
			}
			return base.CanConvertTo (context, destinationType);
		}

		public override object ConvertFrom(ITypeDescriptorContext context, System.Globalization.CultureInfo culture, object value)
		{
			if ( value.GetType() == typeof(string) )
			{
				try
				{
					if ( Const.NotSpecifiedString == (string) value )
					{
						return Const.UNDEFINED_TIME;
					}
					else 
					{
#if DEBUG
						string str = (string) value;
						int month = Convert.ToInt32(str.Substring(0, 2));
						int year = Convert.ToInt32(str.Substring(3, 2));
						year += 2000;
						DateTime dt = new DateTime(year, month, 1);
						return dt;
#else
						string str = (string) value;
						return new DateTime(Convert.ToInt32(str.Substring(3, 2)), Convert.ToInt32(str.Substring(0, 2)), 1);
#endif
					}
				}
				catch
				{
					throw new InvalidCastException();
				}
			}

			return base.ConvertFrom(context, culture, value);
		}

		public override object ConvertTo(ITypeDescriptorContext context, System.Globalization.CultureInfo culture, object value, Type destinationType)
		{
			if ( destinationType == typeof(string) && value is DateTime )
			{				
				if ((DateTime) value == Const.UNDEFINED_TIME)
				{
					return Const.NotSpecifiedString;
				}
				else 
				{
					DateTime dt = (DateTime) value;
					return dt.Month.ToString("D2") + '/' + dt.Year.ToString("D2");
				}
			}

			return base.ConvertTo (context, culture, value, destinationType);
		}		
	}
	#endregion

	#region Percentage Type Editor
	public class PercentageTypeConverter : CaveArtTypeConverter
	{
		public override bool CanConvertFrom(ITypeDescriptorContext context, Type sourceType)
		{
			if (sourceType == typeof(string))
			{
				return true;
			}
			return base.CanConvertFrom (context, sourceType);
		}

		public override bool CanConvertTo(ITypeDescriptorContext context, Type destinationType)
		{
			if (destinationType == typeof(string))
			{
				return true;
			}
			return base.CanConvertTo (context, destinationType);
		}

		public override object ConvertFrom(ITypeDescriptorContext context, System.Globalization.CultureInfo culture, object value)
		{
			if(value.GetType() == typeof(string))
			{
				try
				{
					return decimal.Parse(String.Format("{0,2:N}%", value));
				}
				catch
				{
					throw new InvalidCastException();
				}
			}

			return base.ConvertFrom (context, culture, value);
		}

		public override object ConvertTo(ITypeDescriptorContext context, System.Globalization.CultureInfo culture, object value, Type destinationType)
		{
			if ( destinationType == typeof(string) && value is decimal )
			{				
				return String.Format("{0,2:N}%", value);
			}

			return base.ConvertTo (context, culture, value, destinationType);
		}		
	}
	#endregion

	#region DayTypeEditor and Editor
//	public class DayUITypeEditor : System.Drawing.Design.UITypeEditor 
//	{
//		private	IWindowsFormsEditorService	uiSvc			= null;
//		private System.Windows.Forms.TextBox						days	= null;
//		
//		public override UITypeEditorEditStyle GetEditStyle(ITypeDescriptorContext context) 
//		{
//			if (null != context &&
//				null != context.Instance) 
//			{				
//				return UITypeEditorEditStyle.DropDown;
//			}
//			return base.GetEditStyle(context);
//		}
//
//		public override object EditValue(ITypeDescriptorContext context, IServiceProvider provider, object value) 
//		{
//			if (null != context && 
//				null != context.Instance && 
//				null != provider) 
//			{
//				uiSvc = (IWindowsFormsEditorService) provider.GetService( typeof(IWindowsFormsEditorService) );
//				
//				if (null != uiSvc) 
//				{
//					int OriginalValue = Convert.ToInt32(value);
//					days = new TextBox();
//					days.BorderStyle = BorderStyle.None;					
//					days.Text = value.ToString();
//					
//
//					try
//					{
//						uiSvc.DropDownControl(days);
//						OriginalValue = int.Parse(days.Text);
//						Debug.WriteLine("Original Value after:" + days.Text);
//					}
//					catch
//					{
//						throw new InvalidCastException();
//					}
//					finally
//					{
//						uiSvc.CloseDropDown();						
//					}					
//
//					return OriginalValue;
//				}
//			}
//
//			return value;
//		}
//
//		public override bool GetPaintValueSupported(ITypeDescriptorContext context)
//		{
//			return true;
//		}
//	}

	//TODO: initialize the value before entering a new one,
	// 23 Days   ڿ 24  .
	//Ŀ  ...Days , focus  ϸ ٵ...
	//UITypeEditor Ⱦ   ִ ?
	public class DayTypeConverter : CaveArtTypeConverter
	{
		public override bool CanConvertFrom(ITypeDescriptorContext context, Type sourceType)
			{
				if (sourceType == typeof(string))
				{
					return true;
				}
				return base.CanConvertFrom (context, sourceType);
			}
	
			public override bool CanConvertTo(ITypeDescriptorContext context, Type destinationType)
			{
				if (destinationType == typeof(string))
				{
					return true;
				}
				return base.CanConvertTo (context, destinationType);
			}

		public override object ConvertFrom(ITypeDescriptorContext context, System.Globalization.CultureInfo culture, object value)
		{
			if(value.GetType() == typeof(string))
			{
				try
				{
					return int.Parse(value.ToString());
				}
				catch
				{
					throw new InvalidCastException();
				}
			}

			return base.ConvertFrom (context, culture, value);
		}
		public override object ConvertTo(ITypeDescriptorContext context, System.Globalization.CultureInfo culture, object value, Type destinationType)
		{
			if ( destinationType == typeof(string) && value is int )
			{				
				return value.ToString() + " Day" + ((((int)value) > 1)?"s":"");
			}

			return base.ConvertTo (context, culture, value, destinationType);
		}		
	}
	#endregion

	#region Minute Converter
	public class MinuteTypeConverter : CaveArtTypeConverter
	{
		public override bool CanConvertFrom(ITypeDescriptorContext context, Type sourceType)
		{
			if (sourceType == typeof(string))
			{
				return true;
			}
			return base.CanConvertFrom (context, sourceType);
		}
	
		public override bool CanConvertTo(ITypeDescriptorContext context, Type destinationType)
		{
			if (destinationType == typeof(string))
			{
				return true;
			}
			return base.CanConvertTo (context, destinationType);
		}

		public override object ConvertFrom(ITypeDescriptorContext context, System.Globalization.CultureInfo culture, object value)
		{
			if(value.GetType() == typeof(string))
			{
				try
				{
					return int.Parse(value.ToString());
				}
				catch
				{
					throw new InvalidCastException();
				}
			}

			return base.ConvertFrom (context, culture, value);
		}
		public override object ConvertTo(ITypeDescriptorContext context, System.Globalization.CultureInfo culture, object value, Type destinationType)
		{
			if ( destinationType == typeof(string) && value is int )
			{				
				return value.ToString() + " Minute" + ((((int)value) > 1)?"s":"");
			}

			return base.ConvertTo (context, culture, value, destinationType);
		}		
	}
	#endregion

	#region PasswordUITypeEditor
	public class PasswordUITypeEditor : UITypeEditor 
	{
		public const char PassChar = '';

		private	IWindowsFormsEditorService	uiSvc			= null;
		private TextBox						textPassword	= null;
		
		[SecurityPermissionAttribute(SecurityAction.LinkDemand)]
		public override UITypeEditorEditStyle GetEditStyle(ITypeDescriptorContext context) 
		{
			if (null != context &&
				null != context.Instance) 
			{				
				return UITypeEditorEditStyle.DropDown;
			}
			return base.GetEditStyle(context);
		}

		[SecurityPermissionAttribute(SecurityAction.LinkDemand)]
		public override object EditValue(ITypeDescriptorContext context, IServiceProvider provider, object value) 
		{
			if (null != context && 
				null != context.Instance && 
				null != provider) 
			{
				uiSvc = (IWindowsFormsEditorService) provider.GetService( typeof(IWindowsFormsEditorService) );
				
				if (null != uiSvc) 
				{
					string OriginalValue = (string)value;
					textPassword = new TextBox();
					textPassword.BorderStyle = BorderStyle.None;
					textPassword.PasswordChar = PassChar;
					textPassword.Text = (string) value;

					uiSvc.DropDownControl(textPassword);

					OriginalValue = textPassword.Text;

					uiSvc.CloseDropDown();

					return OriginalValue;
				}
			}

			return value;
		}

		public override bool GetPaintValueSupported(ITypeDescriptorContext context)
		{
			return true;
		}
	}

	public class PasswordTypeConverter : TypeConverter
	{
		public override object ConvertTo(ITypeDescriptorContext context, System.Globalization.CultureInfo culture, object value, Type destinationType)
		{
			if( destinationType == typeof(string) && value is string )
			{
				int length = ((string) value).Length;
				string ret = string.Empty;

				for(int i = 0; i < length; i++)
				{
					ret += '';
				}

				return ret;
			}

			return base.ConvertTo (context, culture, value, destinationType);
		}
	}


	#endregion

#if USE_CURRENY_EDIT
	#region CurrencyUITypeEditor
	public class CurrencyUITypeEditor : UITypeEditor 
	{
		private	IWindowsFormsEditorService	uiSvc		= null;
		private CurrencyEdit				currEdit	= null;

		[SecurityPermissionAttribute(SecurityAction.LinkDemand)]
		public override UITypeEditorEditStyle GetEditStyle(ITypeDescriptorContext context) 
		{
			if (null != context &&
				null != context.Instance) 
			{				
				return UITypeEditorEditStyle.DropDown;
			}
			return base.GetEditStyle(context);
		}

		[SecurityPermissionAttribute(SecurityAction.LinkDemand)]
		public override object EditValue(ITypeDescriptorContext context, IServiceProvider provider, object value) 
		{
			if (null != context && 
				null != context.Instance && 
				null != provider) 
			{
				uiSvc = (IWindowsFormsEditorService) provider.GetService( typeof(IWindowsFormsEditorService) );
				
				if (null != uiSvc) 
				{
					decimal OriginalValue = (decimal) value;
					currEdit = new CurrencyEdit();
					currEdit.BorderStyle = BorderStyle.None;
					currEdit.Value = (decimal) value;
					uiSvc.DropDownControl(currEdit);
					OriginalValue = currEdit.Value;
					uiSvc.CloseDropDown();

					return OriginalValue;
				}
			}

			return value;
		}

		public override bool GetPaintValueSupported(ITypeDescriptorContext context)
		{
			return false;
		}
	}
	#endregion
#endif

	#region FlagListEditor
	public class FlagListEditor : UITypeEditor
	{
		/// <summary>
		/// Internal class used for storing custom data in listviewitems
		/// </summary>
		internal	protected class clbItem
		{
			/// <summary>
			/// Creates a new instance of the <c>clbItem</c>
			/// </summary>
			/// <param name="str">The string to display in the <c>ToString</c> method. 
			/// It will contains the name of the flag</param>
			/// <param name="flag">The integer value of the flag</param>
			/// <param name="tooltip">The tooltip to display in the <see cref="CheckedListBox"/></param>
			public clbItem(string str, ulong flag, string tooltip)
			{
				this.str = str;
				this.val = flag;
				this.tooltip = tooltip;
			}

			private ulong	val;
			private string	str;
			private string	tooltip;

			/// <summary>
			/// Gets the ulong value for this item
			/// </summary>
			public ulong Value
			{
				get{return val;}
			}

			/// <summary>
			/// Gets the tooltip for this item
			/// </summary>
			public string Tooltip
			{
				get{return tooltip;}
			}

			/// <summary>
			/// Gets the name of this item
			/// </summary>
			/// <returns>The name passed in the constructor</returns>
			public override string ToString()
			{
				return str;
			}
		}

		
		protected	bool						handleLostfocus = false;
		protected	IWindowsFormsEditorService	edSvc = null;
		protected	CheckedListBox				clb;
		protected	ToolTip						tooltipControl;

		/// <summary>
		/// Overrides the method used to provide basic behaviour for selecting editor.
		/// Shows our custom control for editing the value.
		/// </summary>
		/// <param name="context">The context of the editing control</param>
		/// <param name="provider">A valid service provider</param>
		/// <param name="value">The current value of the object to edit</param>
		/// <returns>The new value of the object</returns>
		public override object EditValue(ITypeDescriptorContext context, IServiceProvider provider, object value) 
		{
			if (context != null
				&& context.Instance != null
				&& provider != null) 
			{

				edSvc = (IWindowsFormsEditorService)provider.GetService(typeof(IWindowsFormsEditorService));

				if (edSvc != null) 
				{					
					// Create a CheckedListBox and populate it with all the enum values
					clb = new CheckedListBox();
					clb.BorderStyle = BorderStyle.FixedSingle;
					clb.CheckOnClick = true;
					clb.MouseDown += new MouseEventHandler(this.OnMouseDown);
					clb.MouseMove += new MouseEventHandler(this.OnMouseMoved);

					tooltipControl = new ToolTip();
					tooltipControl.ShowAlways = true;

					//TODO: Extract the Tax names
					foreach (string name in Enum.GetNames(context.PropertyDescriptor.PropertyType))
					{
						// Get the enum value
						object enumVal = Enum.Parse(context.PropertyDescriptor.PropertyType, name);
						// Get the int value 
						ulong intVal = (ulong) Convert.ChangeType(enumVal, typeof(ulong));
						
						// Get the description attribute for this field
						System.Reflection.FieldInfo fi = context.PropertyDescriptor.PropertyType.GetField(name);
						DescriptionAttribute[] attrs = (DescriptionAttribute[]) fi.GetCustomAttributes(typeof(DescriptionAttribute), false);

						// Store the the description
						string tooltip = attrs.Length > 0 ? attrs[0].Description : string.Empty;

						// Get the int value of the current enum value (the one being edited)
						ulong intEdited = (ulong) Convert.ChangeType(value, typeof(ulong));

						// Creates a clbItem that stores the name, the int value and the tooltip
						clbItem item = new clbItem(enumVal.ToString(), intVal, tooltip);

						// Get the checkstate from the value being edited
						bool checkedItem = (intEdited & intVal) > 0;

						// Add the item with the right check state
						clb.Items.Add(item, checkedItem);
					}					

					// Show our CheckedListbox as a DropDownControl. 
					// This methods returns only when the dropdowncontrol is closed
					edSvc.DropDownControl(clb);

					// Get the sum of all checked flags
					ulong result = 0;
					foreach (clbItem obj in clb.CheckedItems)
					{
						result += obj.Value;
					}
					
					// return the right enum value corresponding to the result
					return Enum.ToObject(context.PropertyDescriptor.PropertyType, result);
				}
			}

			return value;
		}

		/// <summary>
		/// Shows a dropdown icon in the property editor
		/// </summary>
		/// <param name="context">The context of the editing control</param>
		/// <returns>Returns <c>UITypeEditorEditStyle.DropDown</c></returns>
		public override UITypeEditorEditStyle GetEditStyle(ITypeDescriptorContext context) 
		{
			return UITypeEditorEditStyle.DropDown;			
		}

		/// <summary>
		/// When got the focus, handle the lost focus event.
		/// </summary>
		/// <param name="sender"></param>
		/// <param name="e"></param>
		protected void OnMouseDown(object sender, MouseEventArgs e) 
		{
			if(!handleLostfocus && clb.ClientRectangle.Contains(clb.PointToClient(new Point(e.X, e.Y))))
			{
				clb.LostFocus += new EventHandler(this.ValueChanged);
				handleLostfocus = true;
			}
		}

		/// <summary>
		/// Occurs when the mouse is moved over the checkedlistbox. 
		/// Sets the tooltip of the item under the pointer
		/// </summary>
		/// <param name="sender"></param>
		/// <param name="e"></param>
		protected void OnMouseMoved(object sender, MouseEventArgs e) 
		{			
			int index = clb.IndexFromPoint(e.X, e.Y);
			if(index >= 0)
				tooltipControl.SetToolTip(clb, ((clbItem) clb.Items[index]).Tooltip);
		}

		/// <summary>
		/// Close the dropdowncontrol when the user has selected a value
		/// </summary>
		/// <param name="sender"></param>
		/// <param name="e"></param>
		protected void ValueChanged(object sender, EventArgs e) 
		{
			if (edSvc != null) 
			{
				edSvc.CloseDropDown();
			}
		}
	}
	#endregion

	#region TaxItem
	public class TaxItem
	{
		public TaxItem(string str, bool check)
		{
			this.str = str;
			this.val = check;
		}

		private bool	val;
		private string	str;

		public bool Value
		{
			get { return val; }
			set { val = value; }
		}

		public string Name
		{
			get { return str; }
			set { str = value; }
		}

		public override string ToString()
		{
			return str;
		}
	}
	#endregion

	#region TaxItemCollection
	public class TaxItemCollection : CollectionBase
	{
		public TaxItem this[ int index ]  
		{
			get  
			{
				return( (TaxItem) List[index] );
			}
			set  
			{
				List[index] = value;
			}
		}

		public int Add( TaxItem value )  
		{
			return( List.Add( value ) );
		}

		public int IndexOf( TaxItem value )  
		{
			return( List.IndexOf( value ) );
		}

		public void Insert( int index, TaxItem value )  
		{
			List.Insert( index, value );
		}

		public void Remove( TaxItem value )  
		{
			List.Remove( value );
		}

		public bool Contains( TaxItem value )  
		{
			return( List.Contains( value ) );
		}

		protected override void OnInsert( int index, Object value )  
		{
			if ( value.GetType() != typeof(TaxItem) )
				ThrowException();
		}

		protected override void OnRemove( int index, Object value )  
		{
			if ( value.GetType() != typeof(TaxItem) )
				ThrowException();
		}

		protected override void OnSet( int index, Object oldValue, Object newValue )  
		{
			if ( newValue.GetType() != typeof(TaxItem) )
				ThrowException();
		}

		protected override void OnValidate( Object value )  
		{
			if ( value.GetType() != typeof(TaxItem) )
				ThrowException();
		}

		protected void ThrowException()
		{
			throw new ArgumentException( "value has the type of " + GetType().Name + "." );
		}
	}
	#endregion

	#region TaxCodeEditor
	public class TaxCodeEditor : UITypeEditor
	{
		private	bool						handleLostfocus = false;
		private	IWindowsFormsEditorService	edSvc = null;
		private	CheckedListBox				clb;
		public static TaxItemCollection		TaxItems = new TaxItemCollection();
		
		public override object EditValue(ITypeDescriptorContext context, IServiceProvider provider, object value) 
		{
			if (context != null &&
				context.Instance != null &&
				provider != null) 
			{

				edSvc = (IWindowsFormsEditorService) provider.GetService(typeof(IWindowsFormsEditorService));

				if (edSvc != null) 
				{					
					clb = new CheckedListBox();
					clb.BorderStyle = BorderStyle.None;
					clb.CheckOnClick = true;
					clb.MouseDown += new MouseEventHandler(this.OnMouseDown);
					clb.SelectedIndexChanged +=new EventHandler(SelectedIndexChanged);
					
					TaxCodeEditor.SetCheckStatus(Convert.ToByte(value));
					RefreshCheckListItems();
					edSvc.DropDownControl(clb);

					return TaxCodeEditor.GetCheckStatus();
				}
			}

			return value;
		}

		private void RefreshCheckListItems()
		{
			clb.Items.Clear();
			foreach (TaxItem item in TaxCodeEditor.TaxItems)
			{
				clb.Items.Add(item.Name, item.Value);
			}
		}

		public override UITypeEditorEditStyle GetEditStyle(ITypeDescriptorContext context) 
		{
			return UITypeEditorEditStyle.DropDown;			
		}

		protected void OnMouseDown(object sender, MouseEventArgs e) 
		{
			if (
				!handleLostfocus && 
				clb.ClientRectangle.Contains(clb.PointToClient(new Point(e.X, e.Y)))
				)
			{
				clb.LostFocus += new EventHandler(this.ValueChanged);
				handleLostfocus = true;
			}
		}

		protected void ValueChanged(object sender, EventArgs e) 
		{
			if (edSvc != null) 
			{
				edSvc.CloseDropDown();
			}
		}

		private void SelectedIndexChanged(object sender, EventArgs e)
		{
			int m = ( (CheckedListBox) sender ).SelectedIndex;
			TaxItems[m].Value = !TaxItems[m].Value;
		}

		public static void SetCheckStatus(byte val)
		{
			int mod = 1;
			for (int cx=0; cx<TaxCodeEditor.TaxItems.Count; cx++)
			{
				TaxCodeEditor.TaxItems[cx].Value = (val & mod) > 0;
				mod = mod * 2;
			}
		}

		public static byte GetCheckStatus()
		{
			int code	= 0;
			int mod		= 1;

			foreach (TaxItem item in TaxCodeEditor.TaxItems)
			{
				if (item.Value)
				{
					code += mod;
				}
				mod = mod * 2;
			}

			return (byte) code;
		}
	}
	#endregion

#if REF_CODE
	/*
	#region TaxCodeEditor
	public class TaxCodeEditor : UITypeEditor
	{
		private	bool						handleLostfocus = false;
		private	IWindowsFormsEditorService	edSvc = null;
		private	CheckedListBox				clb;
		public static TaxItemCollection		TaxItems = new TaxItemCollection();
		
		public override object EditValue(ITypeDescriptorContext context, IServiceProvider provider, object value) 
		{
			if (context != null &&
				context.Instance != null &&
				provider != null) 
			{

				edSvc = (IWindowsFormsEditorService) provider.GetService(typeof(IWindowsFormsEditorService));

				if (edSvc != null) 
				{					
					clb = new CheckedListBox();
					clb.BorderStyle = BorderStyle.None;
					clb.CheckOnClick = true;
					clb.MouseDown += new MouseEventHandler(this.OnMouseDown);
					clb.SelectedIndexChanged +=new EventHandler(SelectedIndexChanged);
					
					TaxCodeEditor.SetCheckStatus(Convert.ToByte(value));
					RefreshCheckListItems();
					edSvc.DropDownControl(clb);

					return TaxCodeEditor.GetCheckStatus();
				}
			}

			return value;
		}

		private void RefreshCheckListItems()
		{
			clb.Items.Clear();
			foreach (TaxItem item in TaxCodeEditor.TaxItems)
			{
				clb.Items.Add(item.Name, item.Value);
			}
		}

		public override UITypeEditorEditStyle GetEditStyle(ITypeDescriptorContext context) 
		{
			return UITypeEditorEditStyle.DropDown;			
		}

		protected void OnMouseDown(object sender, MouseEventArgs e) 
		{
			if (
				!handleLostfocus && 
				clb.ClientRectangle.Contains(clb.PointToClient(new Point(e.X, e.Y)))
				)
			{
				clb.LostFocus += new EventHandler(this.ValueChanged);
				handleLostfocus = true;
			}
		}

		protected void ValueChanged(object sender, EventArgs e) 
		{
			if (edSvc != null) 
			{
				edSvc.CloseDropDown();
			}
		}

		private void SelectedIndexChanged(object sender, EventArgs e)
		{
			int m = ( (CheckedListBox) sender ).SelectedIndex;
			TaxItems[m].Value = !TaxItems[m].Value;
		}

		public static void SetCheckStatus(byte val)
		{
			int mod = 1;
			for (int cx=0; cx<TaxCodeEditor.TaxItems.Count; cx++)
			{
				TaxCodeEditor.TaxItems[cx].Value = (val & mod) > 0;
				mod = mod * 2;
			}
		}

		public static byte GetCheckStatus()
		{
			int code	= 0;
			int mod		= 1;

			foreach (TaxItem item in TaxCodeEditor.TaxItems)
			{
				if (item.Value)
				{
					code += mod;
				}
				mod = mod * 2;
			}

			return (byte) code;
		}
	}
	#endregion
	*/
#endif


	#region PrinterSelectEditor
	//	public class PrinterSelectEditor : UITypeEditor
	//	{
	//		/*
	//		private	IWindowsFormsEditorService			edSvc = null;
	//		private ComboBox							combo;
	//
	//		public override object EditValue(ITypeDescriptorContext context, IServiceProvider provider, object value)
	//		{
	//			if (context != null &&
	//				context.Instance != null &&
	//				provider != null) 
	//			{
	//				edSvc = (IWindowsFormsEditorService) provider.GetService(typeof(IWindowsFormsEditorService));
	//
	//				if (edSvc != null) 
	//				{					
	//					combo = new ComboBox();
	//					combo.DropDownStyle = ComboBoxStyle.DropDownList;
	//					RefreshComboItems();
	//					// combo.MouseDown += new MouseEventHandler(this.OnMouseDown);
	//					combo.SelectedIndexChanged += new EventHandler(SelectedIndexChanged);
	//
	//					//if (value
	//					// combo.SelectedIndex = Convert.ToInt32(value);
	//
	//					edSvc.DropDownControl(combo);
	//					return value; // PrinterSettings.InstalledPrinters[combo.SelectedIndex];
	//				}
	//			}
	//
	//			return value;
	//		}
	//
	//		private void RefreshComboItems()
	//		{
	//			combo.Items.Clear();
	//			foreach (string str in PrinterSettings.InstalledPrinters)
	//			{
	//				combo.Items.Add(str);
	//			}
	//		}
	//
	//		public override UITypeEditorEditStyle GetEditStyle(ITypeDescriptorContext context)
	//		{
	//			return UITypeEditorEditStyle.DropDown;
	//		}
	//
	//		protected void ValueChanged(object sender, EventArgs e) 
	//		{
	//			if (edSvc != null) 
	//			{
	//				edSvc.CloseDropDown();
	//			}
	//		}
	//
	//		private void SelectedIndexChanged(object sender, EventArgs e)
	//		{
	//			// int m = ( (ComboBox) sender ).SelectedIndex;
	//			// combo.S
	//			
	//
	//			// TaxItems[m].Value = !TaxItems[m].Value;
	//		}
	//		*/
	//	}
	#endregion
}
